#!/usr/bin/perl -w

# impgraph -- mailgraph-like IMP login/failed status
#
# based on mailgraph by David Schweikert <dws@ee.ethz.ch>
# released under the GNU General Public License

use RRDs;
use POSIX qw(uname);

my $VERSION = 0.3;

my $host = (POSIX::uname())[1];
my $scriptname = 'impgraph.cgi';
my $xpoints = 576;
my $ypoints = 150;
my $ypoints_err = 80;
my $rrd = '/var/db/impgraph/impgraph.rrd'; # path to where the RRD database is
my $tmp_dir = '/tmp/impgraph'; # temporary directory where to store the images

my @graphs = (
	{ title => 'Day Graph',   seconds => 3600*24,        },
	{ title => 'Week Graph',  seconds => 3600*24*7,      },
	{ title => 'Month Graph', seconds => 3600*24*31,     },
	{ title => 'Year Graph',  seconds => 3600*24*365, },
);

my %color = (
	login    => 'DDBB00', # rrggbb in hex
	failed   => '990000',
);

sub graph($$$)
{
	my $range = shift;
	my $file = shift;
	my $title = shift;
	my $step = $range*2/$xpoints;

	my ($graphret,$xs,$ys) = RRDs::graph($file,
		'--imgformat', 'PNG',
		'--width', $xpoints,
		'--height', $ypoints,
		'--start', "-$range",
		'--vertical-label', 'times/min',
		'--title', $title,
		"DEF:login=$rrd:login:AVERAGE",
		"DEF:failed=$rrd:failed:AVERAGE",
		"DEF:mlogin=$rrd:login:MAX",
		"DEF:mfailed=$rrd:failed:MAX",
		"CDEF:rlogin=login,60,*",
		"CDEF:rfailed=failed,60,*",
		"CDEF:rmlogin=mlogin,60,*",
		"CDEF:rmfailed=mfailed,60,*",
		"CDEF:dlogin=login,UN,0,login,IF,$step,*",
		"CDEF:slogin=PREV,UN,dlogin,PREV,IF,dlogin,+",
		"CDEF:dfailed=failed,UN,0,failed,IF,$step,*",
		"CDEF:sfailed=PREV,UN,dfailed,PREV,IF,dfailed,+",
		"AREA:rlogin#$color{login}:Login   ",
		'GPRINT:slogin:MAX:total\: %8.0lf times',
		'GPRINT:rlogin:AVERAGE:average\: %.2lf times/min',
		'GPRINT:rmlogin:MAX:max\: %.0lf times/min\l',
		"AREA:rfailed#$color{failed}:Failed  ",
		'GPRINT:sfailed:MAX:total\: %8.0lf times',
		'GPRINT:rfailed:AVERAGE:average\: %.2lf times/min',
		'GPRINT:rmfailed:MAX:max\: %.0lf times/min\l',
		'HRULE:0#000000',
		'COMMENT:\s',
		'COMMENT:graph created on '.localtime(time).'\r',
	);
	my $ERR=RRDs::error;
	die "ERROR: $ERR\n" if $ERR;
}

sub print_html()
{
	print "Content-Type: text/html\n\n";

	print <<HEADER;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN" "http://www.w3.org/TR/REC-html40/loose.dtd">
<HTML>
<HEAD>
<TITLE>IMP Statistics for $host</TITLE>
</HEAD>
<BODY BGCOLOR="#FFFFFF">
HEADER

	print "<H1>IMP Statistics for $host</H1>\n";
	for my $n (0..$#graphs) {
		print "<H2>$graphs[$n]{title}</H2>\n";
		print "<P><IMG BORDER=\"0\" SRC=\"$scriptname/impgraph_${n}.png\" ALT=\"impgraph\">\n";
	}

	print <<FOOTER;
<table border="0" width="400"><tr><td align="left">
<A href="http://rafan.infor.org/software/impgraph">IMPgraph</A> $VERSION
</td>
<td ALIGN="right">
<a HREF="http://people.ee.ethz.ch/~oetiker/webtools/rrdtool/"><img border="0" src="http://people.ethz.ch/~oetiker/webtools/rrdtool/.pics/rrdtool.gif" alt="" width="120" height="34"></a>
</td></tr></table>
</BODY></HTML>
FOOTER
}

sub send_image($)
{
	my $file = shift;
	-r $file or do {
		print "Content-type: text/plain\n\nERROR: can't find $file\n";
		exit 1;
	};

	print "Content-type: image/png\n";
	print "Content-length: ".((stat($file))[7])."\n";
	print "\n";
	open(IMG, $file) or die;
	my $data;
	print $data while read IMG, $data, 1024;
}

sub main()
{
	if($ENV{PATH_INFO}) {
		my $uri = $ENV{REQUEST_URI};
		$uri =~ s/\/[^\/]+$//;
		$uri =~ s/\//,/g;
		$uri =~ s/(\~|\%7E)/tilde,/g;
		mkdir $tmp_dir, 0777 unless -d $tmp_dir;
		mkdir "$tmp_dir/$uri", 0777 unless -d "$tmp_dir/$uri";
		my $file = "$tmp_dir/$uri$ENV{PATH_INFO}";
		if($ENV{PATH_INFO} =~ /^\/impgraph_(\d+)\.png$/) {
			graph($graphs[$1]{seconds}, $file, $graphs[$1]{title});
		}
		else {
			print "Content-Type: text/plain\n\nERROR: unknown image $ENV{PATH_INFO}\n";
			exit 1;
		}
		send_image($file);
	}
	else {
		print_html;
	}
}

main;
